@@ -113,6 +113,8 @@ extern reply(int, char *, ...);
 extern lreply(int, char *, ...);
 #endif
 
+extern Logwtmp(char *, char *, char *);
+
 #ifdef SRP
 
 #include "t_pwd.h"
@@ -719,7 +721,7 @@ end_login()
 
 	(void) seteuid((uid_t)0);
 	if (logged_in)
-		logwtmp(ttyline, "", "");
+		Logwtmp(ttyline, "", "");
 	pw = NULL;
 	logged_in = 0;
 	guest = 0;
@@ -891,7 +893,7 @@ pass(passwd)
 
 	/* open wtmp before chroot */
 	(void)sprintf(ttyline, "ftp%d", getpid());
-	logwtmp(ttyline, pw->pw_name, remotehost);
+	Logwtmp(ttyline, pw->pw_name, remotehost);
 	logged_in = 1;
 
 	if (guest) {
@@ -1779,7 +1781,7 @@ dologout(status)
 {
 	if (logged_in) {
 		(void) seteuid((uid_t)0);
-		logwtmp(ttyline, "", "");
+		Logwtmp(ttyline, "", "");
 	}
 #ifdef SRP
 	srp_reset ();
