@@ -30,6 +30,23 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
+/*
+%%% portions-copyright-cmetz-97
+Portions of this software are Copyright 1997 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
+*/
+/*
+ * This file has been slightly modified by NRL for use with IPv6+IPsec.
+ * Search for INET6 and/or IPSEC to see the blocks where this happened.
+ * See the NRL Copyright notice for conditions on the modifications.
+ */
+/*
+ * IPv6+IPsec modifications brought up to date for telnet.98.02.16 by
+ * Albert K T Hui <avatar@deva.net>.  All rights reserved.
+ */
 
 #ifndef lint
 static char copyright[] =
@@ -44,6 +61,11 @@ static char sccsid[] = "@(#)telnetd.c	8.4 (Berkeley) 5/30/95";
 #include "telnetd.h"
 #include "pathnames.h"
 
+#ifdef INET6
+#include <netdb.h>
+#include "support.h"
+#endif /* INET6 */
+
 #if	defined(_SC_CRAY_SECURE_SYS) && !defined(SCM_SECURITY)
 /*
  * UNICOS 6.0/6.1 do not have SCM_SECURITY defined, so we can
@@ -168,7 +190,11 @@ char valid_opts[] = {
 main(argc, argv)
 	char *argv[];
 {
+#ifdef INET6
+  union sockaddr_union from;
+#else /* INET6 */
 	struct sockaddr_in from;
+#endif /* INET6 */
 	int on = 1, fromlen;
 	register int ch;
 	extern char *optarg;
@@ -383,6 +409,66 @@ main(argc, argv)
 	if (debug) {
 	    int s, ns, foo;
 	    struct servent *sp;
+#ifdef INET6
+	    struct addrinfo req, *ai, *ai2;
+	    char hbuf[NI_MAXHOST], sbuf[NI_MAXSERV];
+
+	    if (argc > 2) {
+		usage();
+		/* NOT REACHED */
+	    }
+
+	    memset(&req, 0, sizeof(struct addrinfo));
+	    req.ai_flags |= AI_PASSIVE;
+	    req.ai_socktype = SOCK_STREAM;
+
+	    {
+	      char *service = (argc > 0) ? *(argv++) : "telnet";
+	      char *host = (argc > 1) ? *argv : NULL;
+
+	      if (s = getaddrinfo(host, service, &req, &ai)) {
+		fprintf(stderr, "telnetd: %s.%s: %s\n", host, service, gai_strerror(s));
+		usage();
+		/* NOT REACHED */
+	      }
+	    }
+
+	    for (ai2 = ai, s = -1, ns = 1; ns && ai; ai = ai->ai_next) {
+	      if (getnameinfo(ai->ai_addr, ai->ai_addrlen, hbuf, sizeof(hbuf), sbuf, sizeof(sbuf), NI_NUMERICHOST | NI_NUMERICSERV) < 0) {
+		fprintf(stderr, "telnetd: getnameinfo() failed\n");
+		continue;
+	      }
+	      printf("Trying %s.%s...\n", hbuf, sbuf);
+
+	      if (s >= 0)
+		close(s);
+
+	      if ((s = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0) {
+		perror("telnetd: socket");
+		continue;
+	      }
+	      setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof(on));
+
+	      if (bind(s, ai->ai_addr, ai->ai_addrlen)) {
+		perror("bind");
+		continue;
+	      }
+
+	      if (listen(s, 1) < 0) {
+		perror("listen");
+		continue;
+	      }
+	    }
+
+	    if ((ns = accept(s, NULL, 0)) < 0) {
+	      perror("accept");
+	      exit(1);
+	    }
+
+	    dup2(ns, 0);
+	    close(ns);
+	    close(s);
+#else /* INET6 */
 	    static struct sockaddr_in sin = { AF_INET };
 
 	    if (argc > 1) {
@@ -433,6 +519,7 @@ main(argc, argv)
 	    (void) dup2(ns, 0);
 	    (void) close(ns);
 	    (void) close(s);
+#endif /* INET6 */
 #ifdef convex
 	} else if (argc == 1) {
 		; /* VOID*/		/* Just ignore the host/port name */
@@ -529,7 +616,11 @@ main(argc, argv)
 	}
 #endif	/* defined(IPPROTO_IP) && defined(IP_TOS) */
 	net = 0;
-	doit(&from);
+#ifdef INET6
+        doit(&from, fromlen);
+#else /* INET6 */
+        doit(&from);
+#endif /* INET6 */
 	/* NOTREACHED */
 }  /* end of main */
 
@@ -783,6 +874,9 @@ terminaltypeok(s)
 char *hostname;
 char host_name[MAXHOSTNAMELEN];
 char remote_host_name[MAXHOSTNAMELEN];
+#ifdef INET6
+char remote_host_addr[INET6_ADDRSTRLEN];
+#endif /* INET6 */
 
 #define MAXIMLEN 1024
 static char im_buf[MAXIMLEN];
@@ -796,8 +890,14 @@ extern void telnet P((int, int, char *));
 /*
  * Get a pty, scan input lines.
  */
+#ifdef INET6
+doit(who, wholen)
+	struct sockaddr *who;
+	int wholen;
+#else /* INET6 */
 doit(who)
 	struct sockaddr_in *who;
+#endif /* INET6 */
 {
 	char *host, *inet_ntoa();
 	int t;
@@ -846,6 +946,11 @@ doit(who)
 #endif	/* _SC_CRAY_SECURE_SYS */
 
 	/* get name of connected client */
+#ifdef INET6
+	if (getnameinfo(who, wholen, remote_host_name, sizeof(remote_host_name), NULL, 0, 0)) {
+	  fatal(net, "getnameinfo() failed\n");
+	}
+#else /* INET6 */
 	hp = gethostbyaddr((char *)&who->sin_addr, sizeof (struct in_addr),
 		who->sin_family);
 
@@ -864,6 +969,7 @@ doit(who)
 	 * to also do a gethost* and overwrite the static data...
 	 */
 	strncpy(remote_host_name, host, sizeof(remote_host_name)-1);
+#endif /* INET6 */
 	remote_host_name[sizeof(remote_host_name)-1] = 0;
 	host = remote_host_name;
 
@@ -881,7 +987,12 @@ doit(who)
 	*user_name = 0;
 	level = getterminaltype(user_name);
 	setenv("TERM", terminaltype ? terminaltype : "network", 1);
+#ifdef INET6
+	inet_ntop(AF_INET6, &((struct sockaddr_in6 *)who)->sin6_addr, remote_host_addr, sizeof(remote_host_addr));
+	setenv("REMOTE_IPADDRESS", remote_host_addr, 0);
+#else /* INET6 */
 	setenv("REMOTE_IPADDRESS", inet_ntoa(who->sin_addr), 0);
+#endif /* INET6 */
 	setenv("REMOTE_HOSTNAME", remote_host_name, 0);
 
 	/*
